--[[
The Library module containing functions used by R.tester scripts.

Version 30.04.2017
--]]



local rl = {}

-- Just test function
function rl.zombie(str)
    local resultStr = '     (x(x_(X_x(O_o)x_x)_X)x)\n'
    if str then resultStr = str .. resultStr end
    return resultStr
end

-- For the testing and debugging needs
function rl.tmpOut(...)
    local str = '\n\n\n\n\n'
    local arg = {...}
    local n = select('#',...)
    for i = 1, n do
        v = tostring(arg[i])
        str = str .. i .. ' ==> ' .. v .. '\n'
    end
    resultStr = str .. '\n\n\n\n\n' .. resultStr
end

function rl.tmpOutRaw(...)
    local str = ''
    local arg = {...}
    local n = select('#',...)
    for i = 1, n do
        v = tostring(arg[i])
        str = str .. v
    end
    resultStr = str .. resultStr
end

function rl.plural(singular, plural, nomber)
    if nomber == 1 then
        return singular
    else 
        return plural
    end
end

function rl.date()
    return os.date('%d.%m.%Y %X')
end


function rl.createDebugLogFunc(fileName)
    if device.DEBUG then
        if pcall(function(file) io.output(file) end, fileName) then
            io.write (rl.date() .. ' DEBUG OUT ==> ' .. fileName .. '\n\n')
            return function(str) io.write (str .. '\n') end
        else
            return function() return false end
        end
    else
        return function() return true end
    end
end

-- Check if file exists
function rl.fileExists(fileName)
    local f = io.open(fileName,"r")
    if f ~= nil then 
        io.close(f) 
        return true 
    else 
        return false 
    end
end

-- Localization function
function rl.loc(str)
    if not device.LANG then return 'ERROR when using the "rl.loc" localization function. "device.LANG" variable must be set before using this function.' end
    if not libFolder then return 'ERROR when using the "rl.loc" localization function. "LibFolder" variable must be set before using this function.' end
    rl.includeOnce(libFolder .. 'loc_' .. device.LANG .. '.lua')
    if locTable[str] then
        return locTable[str]
    else 
        return str
    end
end


-- Set of functions to load chunks of code for different purposes

function rl.includeChunk(fileName)  
    dofile(fileName) 
end

function rl.addIncludeOnceFunction()
    local includeOnceFileList = {}
    return function(fileName)   
        if not includeOnceFileList[fileName] then 
            includeOnceFileList[fileName] = true
            dofile(fileName)
        end
    end
end
rl.includeOnce = rl.addIncludeOnceFunction()

function rl.protectedLoad(fileName)
    local functionBody, errorMessage = loadfile(fileName)
    if functionBody then 
        return pcall(functionBody) 
    else
        return false, errorMessage
    end
end

-- Returns a table containing all the data from the .ini file or returns nil and error message.
function rl.iniLoad(fileName)
    local file, errorMessage = io.open(fileName, 'r')
    if not file then return nil, errorMessage end
    local data = {}
    local section
    for line in file:lines() do
        local tmpSection = line:match('^%s*%[([^%[%]]+)%]%s*$')
        if tmpSection then
            section = tmpSection
            if not data[section] then data[section] = {} end
        end
        if section then
            local param, value = line:match('^%s*([%w|_]*%s*[%w|_]+)%s*=%s*(.+)%s*$')
            if param and value ~= nil then
                if tonumber(value) then value = tonumber(value)
                elseif value == 'true'  then value = true
                elseif value == 'false' then value = false
                end
                data[section][param] = value
            end
        end
    end
    file:close()
    return data
end


-- Returns the value of the requested parameter of the requested section from the .ini file. Or returns nil and error message.
function rl.iniVal(fileName, reqSection, reqParam)
    local file, errorMessage = io.open(fileName, 'r')
    if not file then return nil, errorMessage end
    local sectionFound
  
    for line in file:lines() do
        local tmpSection = line:match('^%s*%[([^%[%]]+)%]%s*$')
        if tmpSection then
            sectionFound = tmpSection
        end
    if sectionFound == reqSection then
        local value = line:match('^%s*' .. reqParam .. '%s*=%s*(.+)%s*$')
        if value then
            if tonumber(value) then value = tonumber(value)
            elseif value == 'true'  then value = true
            elseif value == 'false' then value = false
            end
            file:close()
            return value
        end
    end
    end
    file:close()
    return nil, 'The requested parameter is not found.'
end


-- Sorts the contents of the table by a key
function rl.pairs(t)
    local a = {}
    for n in pairs(t) do table.insert(a, n) end
        table.sort(a)
        local i = 0 
        local iter = function ()
            i = i + 1
            if a[i] == nil then return nil
            else return a[i], t[a[i]]
        end
    end
    return iter
end

return rl